#!BPY

"""
Name: 'GENARM'
Blender: 234
Group: 'Wizards'
Tooltip: 'Generate a parented armature on a hip of Empty'
"""

######################################################
# GENARM.py v1.0
#
# Script for generating a parented armature on empties
#
#####################################################
# This script generates a parented armature 
# How to use it :
# - Import a bvh in Blender (Fie->Import->BVH)
# - Launch this script (Alt-P)
# - Set up variables : 
#   "hipbonename" the name of the main bone
#   "scale" to resize the armature
# - Press "Create Armature"
######################################################
# Author : Jean-Baptiste PERIN (jb_perin@yahoo.fr)
######################################################



import Blender
from Blender import Mathutils
import math

dicEmptyChild={}

########################################################################
#
#          UTILITY FUNCTIONS FOR HANDLING BONES AND EMPTIES
#
########################################################################


#########
# Cette fonction renvoie la liste des empties attaches a root
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getTree(emp_list, root):
	empties=getAllEmpties()
	chlds = getChildren(root, empties)
	dicEmptyChild[root.getName()]=chlds
	for ch in chlds:
		emp_list.append(ch)
		getTree(emp_list,ch)

#########
# Cette fonction renvoie la liste des empties attaches a root
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getEmpties():
	global hipbonename
	emp_list = []
	root = Blender.Object.Get(hipbonename)
	emp_list.append(root)
	getTree(emp_list, root)
	return emp_list

#########
# Cette fonction renvoie la liste des empties
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getAllEmpties():
	emp_list = []
	objs = Blender.Object.Get()
	for o in objs:
		if o.getType()=="Empty":
			emp_list.append(o)
	return emp_list

#########
# Cette fonction renvoie la liste des empties
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getEmpty(name):
	p = None
	objs = Blender.Object.Get()
	for o in objs:
		if o.getType()=="Empty" and o.getName()==name:
			p = o
	return p

#########
# Cette fonction renvoie la liste des empties parentes a emp
# in  : 
# out : emp_list (List of Object) la liste des objets de type "Empty"
#########
def getChild(emp, emp_list):
	return dicEmptyChild[emp.getName()]


#########
# Cette fonction fournit la liste des enfants d'un empty
# in  : emp (Object) un empty
#       emp_list (List of Object) la liste des empties
# out : children (List of Object) la liste des empties enfants de 'empty'
#########
def getChildren(emp, emp_list):
	children = []
	root_emp = getRootEmpties(emp_list)
	for em in emp_list:
		if (em.getName() != emp.getName()) and (em not in root_emp):
			if (em.getParent().getName() == emp.getName()):
				children.append(em)
	return children



#########
# Cette fonction renvoie la liste des empties n'ayant pas de parent
# in  : emp_list (List) une liste d'empties
# out : root (List) le (ou les) empty n'ayant pas de parent
#########
def getRootEmpties(emp_list):
	root = []
	for em in emp_list:
		if em.getParent() == None:
			root.append(em)
	return root






########################################################################
#
# FUNCTIONS FOR COMPUTING POSITION AND ROTATION OF BONES 
#
########################################################################

#########
# Cette fonction 
# in  : 
# out : 
#########
def computeScaledPos(vec):
	global scalef
	vec2 = Mathutils.Vector([vec[0]*scalef, vec[1]*scalef, vec[2]*scalef])
	return vec2


########################################################################
#
#            FUNCTIONS FOR CREATING AND MOVING ARMATURES
#
########################################################################

#########
# Cette fonction 
# in  : 
# out : 
#########
def createBone (armature, empty, bone, empties):
	children = getChildren(empty, empties)
	print children
	for ch in children:
		if len(children) >= 2:
			bonename = empty.getName()[1:len(empty.getName())]+'_'+ch.getName()[1:len(ch.getName())]
		else :
			bonename = empty.getName()[1:len(empty.getName())]
		b=Blender.Armature.Bone.New(bonename)
		b.setHead(computeScaledPos(empty.getMatrix('worldspace').translationPart()))
		b.setTail(computeScaledPos(ch.getMatrix('worldspace').translationPart()))
		b.setParent(bone)
		armature.addBone(b)
		createBone(armature, ch, b, empties)

	

#########
# Cette fonction fabrique une arma
# in  : 
# out : 
#########
def createArmature (rootEmpty, empties):
	armData = Blender.Armature.New('monArmature')
	children = getChildren(rootEmpty, empties)
	for ch in children:
		b=Blender.Armature.Bone.New(rootEmpty.getName()[1:len(rootEmpty.getName())] + ch.getName()[1:len(ch.getName())])
		b.setHead(computeScaledPos(rootEmpty.getMatrix('worldspace').translationPart()))
		b.setTail(computeScaledPos(ch.getMatrix('worldspace').translationPart()))
		armData.addBone(b)
		createBone(armData, ch, b, empties)
	return armData




########################################################################
#
#                  MAIN PROGRAM
#
########################################################################

def Main():

	global hipbonename


	print "*****START*****"


	##-----------
	## Positionnement des empties
	##-----------
	em0 = Blender.Object.Get(hipbonename)

	Blender.Redraw()



	##-----------
	## Creation de l'armature et des os
	##-----------

	lesEmpties = getEmpties()
	armData = createArmature(em0, lesEmpties)

	##-----------
	## Ajout de l'armature a la scene
	##-----------
	armObj=Blender.Object.New('Armature', 'OBArmature')

	armObj.link(armData)
	scn = Blender.Scene.getCurrent()
	scn.link (armObj)
	Blender.Redraw()

	##-----------
	## Enregistrement de la position  de l'armature
	##-----------

	#bones = armData.getBones()
	#for bo in bones:
	#	bo.setPose([Blender.Armature.Bone.ROT, Blender.Armature.Bone.LOC]) 

	#Blender.Redraw()



	print "*****END*****"


########################################################################
#
#            GUI FUNCTIONS AND VARIABLES
#
########################################################################

HBName = Blender.Draw.Create(0)
ScaleF =  Blender.Draw.Create(0)
Msg = ' '

def event (evt, val):
	if evt == Blender.Draw.ESCKEY:
		Blender.Draw.Exit()
	return

def button_event(evt):
	global BName, ScaleF, Msg
	global hipbonename, scalef
	if evt==1:
		scalef= eval(str(ScaleF.val))
		print "scalef =", scalef
		hipbonename = HBName.val
		ob = getEmpty(hipbonename)
		if (ob!=None): 
			if ob.getParent()!=None:
				Msg = 'Empty '+hipbonename+ ' is not a root bone.'
			else: 
				if (0.0 > scalef):
					Msg = 'Scale factor must be greater than 0'
				else:
					Blender.Draw.Exit()
					Main()
		else:
			Msg = 'Empty '+ hipbonename+ ' not found'
				
		Blender.Draw.Redraw(1)

def GUI():
	global HBName, ScaleF, Msg
	Blender.BGL.glClearColor(0,0,1,1)
	Blender.BGL.glClear(Blender.BGL.GL_COLOR_BUFFER_BIT)
	Blender.BGL.glColor3f(1,1,1)
	Blender.BGL.glRasterPos2i(20,150)
	Blender.Draw.Text ("ARMATURE GENERATOR v1.0 by Jean-Baptiste PERIN", 'normal')
	ScaleF = Blender.Draw.Number("Scale: ", -1, 20, 50, 250, 20, 1.0, 0.0, 10.0,  'Scale Factor')
	HBName = Blender.Draw.String("HipBoneName: ", -1, 20, 75, 250, 20, '_Hips', 100)
	Blender.Draw.Toggle("Create Armature", 1, 20, 10, 100, 20, 0, "Create Armature")
	Blender.BGL.glRasterPos2i(20,40)
	Blender.Draw.Text (Msg, 'normal')


Blender.Draw.Register(GUI, event, button_event)